from dataclasses import dataclass


@dataclass
class Sector:
    name: str = ""
    companies: list[str] = None
    sub_sectors: list['SubSector'] = None



@dataclass
class SubSector:
    name: str = ""
    companies: list[str] = None



import requests
from bs4 import BeautifulSoup
from kap_sdk._search_oid import _search_oid
from kap_sdk.models.company import Company
import zipfile
import io
import pandas as pd
from io import StringIO
from typing import List, Dict, Union
import pandas as pd

DOWNLOAD_URL = "https://www.kap.org.tr/tr/api/home-financial/download-file/"


def _download_xls(mkkMemberOid: str, year: str) -> str:
    # Burada dosyayı indiriyor
    data = requests.get(f"{DOWNLOAD_URL}{mkkMemberOid}/{year}/T", stream=False)
    data.raise_for_status()
    return data.content



def _extract_data(path: str, price: float, table_indices: List[int] = [1, 300, 453]) -> List[Dict[str, Union[str, float]]]:
    tables = pd.read_html(path)
    selected_tables = [tables[i].iloc[:, [1, 3]].dropna().reset_index(drop=True) for i in table_indices if i < len(tables)]

    concat_table = pd.concat(selected_tables, ignore_index=True)
    concat_table.columns = ['key', 'value']

    def clean_value(value: str, price: float) -> Union[float, None]:
        try:
            cleaned_value = value.strip().replace(".", "").replace(",", ".")
            return float(cleaned_value) * price
        except ValueError:
            print(f"Could not convert value: {value}")
            return None

    concat_table['value'] = concat_table['value'].apply(lambda x: clean_value(x, price))
    concat_table.dropna(inplace=True)

    extracted_data = []
    for index, row in concat_table.iterrows():
        extracted_data.append({
            "key": row['key'].strip(),
            "value": row['value']
        })

    return extracted_data




def _find_financial_header_title(data: str) -> dict:
    soup = BeautifulSoup(data, 'html.parser')
    header = soup.find('table', {'class': 'financial-header-table'})
    row_title = header.find_all('tr')[1].find_all(
        "td")[1].text.strip().lower().replace(" ", "_")
    row_price = header.find_all('tr')[0].find_all(
        "td")[1].text.strip().replace("TL", "").replace(".", "")

    try:
        price = float(row_price)
    except ValueError:
        price = 1.0

    return {
        "title": row_title,
        "price": price
    }


async def get_financial_report(company: Company, year: str = "2023") -> dict:
    oid = _search_oid(company)
    content = _download_xls(oid, year=year)
    try:
        zip_file = io.BytesIO(content)
        with zipfile.ZipFile(zip_file, 'r') as zip_ref:
            files = zip_ref.namelist()
            if len(files) == 0:
                raise ValueError(
                    f"No found {company.code} financial report for {year}")
            extracted_data = {}
            for file_name in zip_ref.namelist():
                if file_name.endswith('.xls'):
                    with zip_ref.open(file_name) as file:
                        data = file.read()
                    meta = _find_financial_header_title(data)
                    period = f"period_{file_name.split('_')[-1]}_{meta['title']}"
                    period = period.replace('.xls', '')
                    extracted_data[period] = _extract_data(
                        StringIO(data.decode('utf-8')), meta['price'])
    except Exception as e:
        print(f"Error extracting financial report: {e}")
        raise e
    finally:
        pass

    return extracted_data



import asyncio

from kap_sdk.kap_client import KapClient
import random
import json

async def sample_get_company():
    client = KapClient()
    company = await client.get_company("BIMAS")
    message = f"Sample get company: {company}"
    print(message)

async def sample_get_company_info():
    client = KapClient()
    company = await client.get_company("BIMAS")
    info = await client.get_company_info(company)
    message = f"Sample get company info: {info}"
    print(message)

async def sample_get_financial_report():
    client = KapClient()
    company = await client.get_company("BIMAS")
    report = await client.get_financial_report(company, "2025")
    with open("financial_report.json", "w") as f:
        f.write(json.dumps(report, indent=4, ensure_ascii=False))
    message = f"Sample get financial report: {report}"
    print(message)

async def sample_get_indices():
    client = KapClient()
    indices = await client.get_indices()
    message = f"Sample get indices: {indices}"
    print(message)


async def sample_get_announcements_by_company():
    client = KapClient()
    company = await client.get_company("BIMAS")
    announce = await client.get_announcements(company)
    message = f"Sample get announcements: {announce}"
    print(message)

async def sample_get_announcements():
    client = KapClient()
    announcements = await client.get_announcements()
    message = f"Sample get announcements: {announcements}"
    print(message)


async def sample_get_sectors():
    client = KapClient()
    sectors = await client.get_sectors()
    message = f"Sample get sectors: {sectors}"
    print(message)

async def main():
    #await sample_get_company()
    #await sample_get_company_info()
    await sample_get_financial_report()
    #await sample_get_indices()
    #await sample_get_announcements_by_company()
    #await sample_get_announcements()


if __name__ == "__main__":
    asyncio.run(main=main())






from dataclasses import dataclass


@dataclass
class CompanyInfo:
    address: str = ""
    mail: list[str] = None
    website: str = ""
    companys_duration: str = ""
    independent_audit_firm: str = ""
    indices: list[str] = None
    sectors: list[str] = None
    equity_market: str = ""



class AnnouncementType:
    MaterialEventDisclosure = "ODA"
    FinancialStatement = "FR"
    RegulatoryAuthorityAnnouncements = "DUY"
    Other = "DG"
    Corporate_Actions = "CA"


class MemberType:
    BistCompanies = "IGS"
    InvestmentFirms = "YK"
    PortfolioManagementCompanies = "PYS"
    RegulatoryAuthorities = "DDK"
    OtherKAPMembers = "DG"

    @staticmethod
    def default():
        return [
            MemberType.BistCompanies,
            MemberType.RegulatoryAuthorities,
        ]


class FundType:
    ETF = "BYF"
    MutualFond = "YF"
    PensionFund = "EYF"
    AutoEnrolmentSystemPensionFund = "OKS"
    ForeignFund = "YYF"
    AssetFinanceFund = "VFF"
    HousingFinanceFund = "KFF"
    RealEstateInvestmentFund = "GMF"
    VentureCapitalInvestmentFund = "GSF"
    ProjectFinanceFund = "PFF"

    @staticmethod
    def default():
        return [
            FundType.ETF,
            FundType.RealEstateInvestmentFund,
            FundType.VentureCapitalInvestmentFund,
            FundType.ProjectFinanceFund,
        ]



from kap_sdk.models.company import Company
import requests

URL = "https://www.kap.org.tr/tr/api/member/filter/"

def _search_oid(company: Company) -> dict:
    response = requests.get(f"{URL}{company.code}")
    response.raise_for_status()
    oid = response.json()[0]["mkkMemberOid"]
    if not oid:
        raise Exception(f"Company {company.code} not found.")
    return oid



import logging
from bs4 import BeautifulSoup
from pyppeteer import launch
from kap_sdk.models.sectors import Sector, SubSector

_URL ="https://www.kap.org.tr/tr/Sektorler"

async def scrape_sectors() -> list[Sector]:
    sectors = []
    browser = None

    try:
        browser = await launch(
            handleSIGINT = "false",
            handleSIGTERM = "false",
            handleSIGHUP = "false",
        )
        page = await browser.newPage()

        await page.goto(_URL, {"waitUntil": "domcontentloaded"})
        await page.waitForSelector('#sectorsTable', timeout=10000)
        content = await page.content()
        soup = BeautifulSoup(content, 'html.parser')
        table = soup.find(id='sectorsTable')

        for row in table.find_all('tr')[1:]:
            cols = row.find_all('td')
            if len(cols) == 1:
                spans = cols[0].find_all('span')
                sector_name = spans[0].text.strip() if spans else None
                if sector_name:
                    sector = Sector(name=sector_name, sub_sectors=[],companies=[])
                    sectors.append(sector)
                else:
                    sub_sector_name = cols[0].text.strip()
                    if "Kayıt Bulunmadı" in sub_sector_name:
                        continue

                    sub_sector = SubSector(
                        name=sub_sector_name,
                        companies=[]
                    )
                    last_sector : Sector = sectors[-1]
                    if last_sector:
                        last_sector.sub_sectors.append(sub_sector)

            elif len(cols) == 4:
                code = cols[1].text.strip()
                last_sector: Sector = sectors[-1]
                if len(last_sector.sub_sectors) > 0:
                    last_sub_sector: SubSector = last_sector.sub_sectors[-1]
                    last_sub_sector.companies.append(code)
                else:
                    last_sector.companies.append(code)

    except Exception as e:
        logging.error(f"Scraping failed: {e}")
        raise e
    finally:
        if browser:
            await browser.close()
    return sectors





import os
import tempfile
from kap_sdk.models.announcement_type import AnnouncementType, FundType, MemberType
import datetime
from datetime import datetime, timedelta
import requests
import diskcache
from kap_sdk._companies import scrape_companies
from kap_sdk._indices import scrape_indices
from kap_sdk._company_info import scrape_company_info
from kap_sdk._financial_report import get_financial_report
from kap_sdk._search_oid import _search_oid
from kap_sdk.models.announcement_type import AnnouncementType, FundType, MemberType
from kap_sdk.models.company import Company
from kap_sdk.models.disclosure import Disclosure, DisclosureBasic, DisclosureDetail
from kap_sdk.models.indices import Indice
from kap_sdk.models.company_info import CompanyInfo
from kap_sdk.models.sectors import Sector
from kap_sdk._sectors import scrape_sectors
from typing import Optional


_CACHE_DIR = os.path.join(tempfile.gettempdir(), "kap_cache")
os.makedirs(_CACHE_DIR, exist_ok=True)


class KapClient:

    def __init__(
        self,
        cache_expiry=3600
    ):
        self.cache = diskcache.Cache(_CACHE_DIR)
        self.cache_expiry = cache_expiry

    async def get_companies(self) -> list[Company]:
        key = "companies"
        cached_companies = self.cache.get(key=key)
        if cached_companies:
            return cached_companies
        companies = await scrape_companies()
        self.cache.set(key, companies, expire=self.cache_expiry)
        return companies

    async def get_indices(self) -> list[Indice]:
        key = "indices"
        cached_indices = self.cache.get(key=key)
        if cached_indices:
            return cached_indices
        indices = await scrape_indices()
        self.cache.set(key, indices, expire=self.cache_expiry)
        return indices

    async def get_company(self, code: str) -> Optional[Company]:
        companies = await self.get_companies()
        for company in companies:
            if company.code == code:
                return company
        return None

    async def get_indice(self, code: str) -> Optional[Indice]:
        indices = await self.get_indices()
        for indice in indices:
            if indice.code == code:
                return indice
        return None

    async def get_company_info(self, company: Company) -> Optional[CompanyInfo]:
        key = f"infos_{company.code}"
        cached_company_info = self.cache.get(key=key)
        if cached_company_info:
            return cached_company_info
        company_info = await scrape_company_info(company)
        self.cache.set(key, company_info, expire=self.cache_expiry)
        return company_info

    async def get_financial_report(self, company: Company, year: str = "2023") -> dict:
        key = f"financial_report_{company.code}_{year}"
        cached_financial_report = self.cache.get(key=key)
        if cached_financial_report:
            return cached_financial_report
        financial_report = await get_financial_report(company=company, year=year)
        self.cache.set(key, financial_report, expire=self.cache_expiry)
        return financial_report

    async def get_announcements(
        self,
        company: Company = None,
        fromdate=datetime.today().date() - timedelta(days=30),
        todate=datetime.today().date(),
        disclosure_type: list[AnnouncementType] = None,
        fund_types: list[FundType] = FundType.default(),
        member_types: list[MemberType] = MemberType.default(),
    ) -> list[Disclosure]:
        oid = None
        if company:
            oid = _search_oid(company)

        data = {
            "fromDate": fromdate.strftime("%d.%m.%Y"),
            "toDate": todate.strftime("%d.%m.%Y"),
            "disclosureType": disclosure_type,
            "fundTypes": fund_types,
            "memberTypes": member_types,
            "mkkMemberOid": oid,
        }
        response = requests.post(
            "https://www.kap.org.tr/tr/api/disclosure/list/main", json=data)
        response.raise_for_status()
        json_data = response.json()
        disclosures = [Disclosure(
            disclosureBasic=DisclosureBasic(**item["disclosureBasic"]),
            disclosureDetail=DisclosureDetail(**item["disclosureDetail"])
        ) for item in json_data]
        return disclosures

    async def get_sectors(self) -> list[Sector]:
        key = "sectors"
        cached_sectors = self.cache.get(key=key)
        if cached_sectors:
            return cached_sectors
        sectors = await scrape_sectors()
        self.cache.set(key, sectors, expire=self.cache_expiry)
        return sectors

    def clear_cache(self):
        self.cache.clear()



from dataclasses import dataclass
from typing import Optional, List

@dataclass
class DisclosureBasic:
    title: str
    mkkMemberOid: str
    companyTitle: str
    stockCode: Optional[str]
    relatedStocks: Optional[str]
    disclosureClass: str
    disclosureType: str
    disclosureCategory: str
    publishDate: str
    disclosureId: str
    disclosureIndex: int
    summary: str
    attachmentCount: int
    year: Optional[int]
    donem: Optional[str]
    period: str
    hasMultiLanguageSupport: str
    fundType: Optional[str]
    isLate: bool
    relatedDisclosureOid: Optional[str]
    senderType: Optional[str]
    isChanged: Optional[bool]
    isBlocked: bool

@dataclass
class DisclosureDetail:
    ftNiteligi: Optional[str]
    decimalDegree: Optional[float]
    opinion: Optional[str]
    opinionType: Optional[str]
    auditType: Optional[str]
    mainDisclosureDocumentId: str
    opinionMemberTitle: Optional[str]
    relatedDisclosureIndex: Optional[int]
    oldKap: bool
    fundOid: Optional[str]
    senderTypes: Optional[str]
    nonInactiveCount: Optional[int]
    blockedDescription: Optional[str]
    memberType: Optional[str]

@dataclass
class Disclosure:
    disclosureBasic: DisclosureBasic
    disclosureDetail: DisclosureDetail
